#!/usr/bin/python
# -*- coding: utf-8 -*-

from logger import Logger
from singleton import Singleton
import bottle
import json
import os


class Certificate(metaclass=Singleton):
    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса получения сертификатов с сервера')
        self.__updateCases()

        servicestub.route('/ACC/rest/v1/cards/<certificateNumber>', method='GET', callback=self.certificateByNumber)
        servicestub.route('/ACC/rest/v1/assets<transactionData:re:.*>', method='POST', callback=self.transaction)
        servicestub.route('/ACC/rest/v1/assets/<sessionId:re:(_cash.*?)>/rollback<transactionData:re:(.*?$)>', method='PUT', callback=self.transactionRollback)

    def certificateByNumber(self, certificateNumber=''):
        self.log.info("Запрос данных о карте по номеру карты: '%s'" % certificateNumber)
        methodPath = '/v1/cards/' + certificateNumber

        if methodPath not in self.cases:
            methodPath = '/v1/cards/<certificateNumber>'

        return self.__getResult(methodPath)

    def transaction(self, transactionData=''):
        self.log.info("Создание транзакции по оплате: данные транзакции: '%s'" % transactionData)
        methodPath = '/v1/assets/' + transactionData

        if methodPath not in self.cases:
            methodPath = '/v1/assets/<transactionData>'

        return self.__getResult(methodPath)

    def transactionRollback(self, sessionId='', transactionData=''):
        self.log.info("Создание транзакции отмены оплаты: информация о сессии: '%s', данные транзакции: '%s'" % (sessionId, transactionData))
        methodPath = '/v1/assets/' + sessionId + '/rollback/' + transactionData

        if methodPath not in self.cases:
            methodPath = '/v1/assets/<sessionId>rollback/<transactionData>'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        if not len(self.cases[methodPath]):
            self.log.warn('В кейсах не осталось значений')
            return None

        body = json.dumps(self.cases[methodPath][0].get('response'))
        status = self.cases[methodPath][0]['status']
        response = bottle.HTTPResponse(status=status if status else 200, body=body)
        response.add_header("Content-Type", "application/json")

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodPath][0]:
            return response

        self.cases[methodPath][0]['count'] -= 1

        if self.cases[methodPath][0]['count'] == 0:
            del self.cases[methodPath][0]

        return response
    
    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'certificate.json'), 'r') as cases:
                self.cases = json.load(cases, encoding='utf-8')
        except Exception as e:
            self.log.error(str(e))


instance = Certificate()
