#!/usr/bin/python
# -*- coding: utf-8 -*-
import time

from logger import Logger
from singleton import Singleton
import bottle
import json
import os


class Crpt(metaclass=Singleton):
    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса crpt')
        self.__updateCases()

        servicestub.route('/api/v4/true-api/codes/check', method='POST', callback=self.check)

    def check(self):
        self.__updateCases()

        self.log.info('Проверка маркировки')

        params = dict(bottle.request.query.decode())

        methodPath = '/api/v4/true-api/codes/check'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath][0].get('timeout', 0))

        body = json.dumps(self.cases[methodPath][0].get('response'))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")

        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'crpt.json'), 'r') as cases:
                self.cases = json.load(cases, encoding='utf-8')
        except Exception as e:
            self.log.error(str(e))


instance = Crpt()
