#!/usr/bin/python
# -*- coding: utf-8 -*-

from logger import Logger
from singleton import Singleton
import configparser
import bottle
import json
import os


class UzumPay(metaclass=Singleton):
    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса')

        conf = configparser.RawConfigParser()
        conf.read(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'config.ini'))
        self.servicePath = conf.get('uzumpay', 'path') if conf.has_option('uzumpay', 'path') else '/api/apelsin-pay/merchant'

        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cases.json'), 'r') as cases:
                self.cases = json.load(cases, encoding='utf-8')
        except Exception as e:
            self.log.error(str(e))

        servicestub.route(os.path.join(self.servicePath, 'payment'), method='POST', callback=self.payment)
        servicestub.route(os.path.join(self.servicePath, 'payment/status'), method='POST', callback=self.status)
        servicestub.route(os.path.join(self.servicePath, 'payment/reversal'), method='POST', callback=self.reversal)
        servicestub.route(os.path.join(self.servicePath, 'partial/reversal'), method='POST', callback=self.partialReversal)
        servicestub.route(os.path.join(self.servicePath, 'payment/fiscal'), method='POST', callback=self.fiscal)

    def payment(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('payment')

    def status(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('payment/status')

    def reversal(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('payment/reversal')

    def partialReversal(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('partial/reversal')

    def fiscal(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('payment/fiscal')

    def createResponse(self, methodName):
        if methodName not in self.cases:
            raise bottle.HTTPError(400, 'Метод "%s" не найден' % methodName)

        if not len(self.cases[methodName]):
            self.log.warn('В кейсах не осталось значений')
            return None

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodName][0]:
            return json.dumps(self.cases[methodName][0].get('response'))

        self.cases[methodName][0]['count'] -= 1
        response = json.dumps(self.cases[methodName][0].get('response'))

        if self.cases[methodName][0]['count'] == 0:
            del self.cases[methodName][0]

        return response

instance = UzumPay()
